from djangoldp.filters import LDPPermissionsFilterBackend


class InboxFilterBackend(LDPPermissionsFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if not request.user.is_anonymous:
            return queryset.filter(user=request.user)
        else:
            from djangoldp_notification.models import Notification
            return Notification.objects.none()


class SubscriptionsFilterBackend(LDPPermissionsFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if request.method == "OPTIONS":
            return queryset
        else:
            return super().filter_queryset(request, queryset, view)



import uuid
from rest_framework.test import APITestCase, APIClient

from djangoldp.serializers import LDListMixin, LDPSerializer
from djangoldp_account.models import LDPUser
from djangoldp_notification.models import Subscription


class TestSubscription(APITestCase):
    circle_user1_url = "http://localhost:8000/circles/1/"
    circle_user2_url = "http://localhost:8000/circles/2/"

    def _get_random_user(self):
        return LDPUser.objects.create(email='{}@test.co.uk'.format(str(uuid.uuid4())), first_name='Test',
                                      last_name='Test', username=str(uuid.uuid4()))

    def _auth_as_user(self, user):
        self.client.force_authenticate(user=user)

    def setUpLoggedInUser(self):
        self.user = self._get_random_user()
        self._auth_as_user(self.user)

    def setUp(self):
        self.client = APIClient()
        LDListMixin.to_representation_cache.reset()
        LDPSerializer.to_representation_cache.reset()

        self.user1 = self._get_random_user()
        Subscription.objects.create(object=self.circle_user1_url, inbox="http://testserver/users/karl_marx/inbox/")

        self.user2 = self._get_random_user()
        Subscription.objects.create(object=self.circle_user2_url,
                                    inbox="http://testserver/users/piotr_kropotkine/inbox/")

    def test_not_logged_fails(self):
        response = self.client.get("/subscriptions/")
        self.assertEqual(response.status_code, 403)

    def test_logged_in_succeeds(self):
        self._auth_as_user(self.user2)
        response = self.client.get("/subscriptions/").data.get("ldp:contains")
        self.assertEqual(len(response), 2)
        response = response[1]
        self.assertEqual(response["object"], self.circle_user2_url)
        self.assertEqual(response["inbox"], "http://testserver/users/piotr_kropotkine/inbox/")
        self.assertIn({'mode': {'@type': 'view'}}, response["permissions"])
        self.assertIn({'mode': {'@type': 'delete'}}, response["permissions"])



from django.contrib.auth import get_user_model

from djangoldp.permissions import LDPPermissions
from djangoldp_notification.filters import InboxFilterBackend, SubscriptionsFilterBackend
from rest_framework.reverse import reverse


class InboxPermissions(LDPPermissions):
    filter_backends = [InboxFilterBackend]


class SubscriptionsPermissions(LDPPermissions):
    filter_backends = [SubscriptionsFilterBackend]

    def has_permission(self, request, view):
        if request.user.is_anonymous and not request.method == "OPTIONS":
            return False

        if request.method in ["GET", "PATCH", "DELETE", "PUT"]:
            return True

        return super().has_permission(request, view)

    def has_object_permission(self, request, view, obj):
        if request.user.is_anonymous and not request.method == "OPTIONS":
            return False

        reverse_path_key = "{}-notification-list".format(get_user_model()._meta.object_name.lower())
        user_inbox = reverse(reverse_path_key, kwargs={"slug": request.user.slug}, request=request)
        if obj.inbox == user_inbox:
            return True

        return False



from django.apps import apps
from django.conf import settings
from django.db import models
from djangoldp.models import Model
from djangoldp_notification.models import send_request, Subscription
from djangoldp_notification.middlewares import MODEL_MODIFICATION_USER_FIELD

class technical_user:
  urlid = settings.BASE_URL

def add_arguments(parser):
  parser.add_argument(
    "--ignore-subscriptions",
    default=False,
    nargs="?",
    const=True,
    help="Ignore subscriptions related check",
  )
  parser.add_argument(
    "--send-subscription-notifications",
    default=False,
    nargs="?",
    const=True,
    help="Send an update notification for every local resources to every subscribers",
  )

def check_integrity(options):
  print('---')
  print("DjangoLDP Notification")
  if(not options["ignore_subscriptions"]):

    models_list = apps.get_models()

    resources = set()
    resources_map = dict()

    for model in models_list:
      for obj in model.objects.all():
        if hasattr(obj, "urlid"):
          if(obj.urlid):
            if(obj.urlid.startswith(settings.BASE_URL)):
              if(type(obj).__name__ != "ScheduledActivity" and type(obj).__name__ != "LogEntry" and type(obj).__name__ != "Activity"):
                resources.add(obj.urlid)
                resources_map[obj.urlid] = obj
    
    print("Found "+str(len(resources_map))+" local resources on "+str(len(models_list))+" models for "+str(Subscription.objects.count())+" subscribers")

    if(options["send_subscription_notifications"]):
      sent=0
      for resource in sorted(resources):
        resource = resources_map[resource]
        recipients = []
        try:
          url_container = settings.BASE_URL + Model.container_id(resource)
          url_resource = settings.BASE_URL + Model.resource_id(resource)
        except NoReverseMatch:
          continue
        for subscription in Subscription.objects.filter(models.Q(object=url_resource) | models.Q(object=url_container)):
          if subscription.inbox not in recipients and not subscription.is_backlink:
            # I may have configured to send the subscription to a foreign key
            if subscription.field is not None and len(subscription.field) > 1:
              try:
                resource = getattr(resource, subscription.field, resource)

                # don't send notifications for foreign resources
                if hasattr(resource, 'urlid') and Model.is_external(resource.urlid):
                    continue

                url_resource = settings.BASE_URL + Model.resource_id(resource)
              except NoReverseMatch:
                continue
              except ObjectDoesNotExist:
                continue
            setattr(resource, MODEL_MODIFICATION_USER_FIELD, technical_user)
            try:
              send_request(subscription.inbox, url_resource, resource, False)
              sent+=1
            except:
              print("Failed to create subscription activity for "+url_resource)
            recipients.append(subscription.inbox)
      print("Sent "+str(sent)+" activities to "+str(Subscription.objects.count())+" subscribers")
      print("Depending on your server configuration, you may have to restart the ActivityQueue or the server to properly process em.")
    else:
      print("Send an update notification for all those resources to all of their subscribers with  `./manage.py check_integrity --send-subscription-notifications`")
  else:
    print("Ignoring djangoldp-notification checks")


# Generated by Django 2.2.18 on 2021-02-18 10:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0010_notificationsetting'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='notification',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control'], 'ordering': ['-date']},
        ),
        migrations.AlterModelOptions(
            name='subscription',
            options={'default_permissions': ['add', 'change', 'delete', 'view', 'control']},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-06-02 10:35
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_notification', '0005_auto_20200505_1733'),
    ]

    operations = [
        migrations.AddField(
            model_name='subscription',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='djangoldp_notification.Subscription'),
        ),
    ]
